Stan Math Library: stan/math/prim/scal/prob/inv_gamma_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

inv_gamma_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_INV_GAMMA_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_INV_GAMMA_LOG_HPP
3
4 #include <boost/random/gamma_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
9 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
10 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
11 #include <stan/math/prim/scal/err/check_not_nan.hpp>
12 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
13 #include <stan/math/prim/scal/fun/constants.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/fun/lgamma.hpp>
16 #include <stan/math/prim/scal/fun/gamma_q.hpp>
17 #include <stan/math/prim/scal/fun/digamma.hpp>
18 #include <stan/math/prim/scal/meta/length.hpp>
19 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
20 #include <stan/math/prim/scal/meta/VectorView.hpp>
21 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
22 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
23 #include <stan/math/prim/scal/meta/return_type.hpp>
24 #include <stan/math/prim/scal/fun/grad_reg_inc_gamma.hpp>
25 #include <stan/math/prim/scal/meta/include_summand.hpp>
26 #include <cmath>
27
28 namespace stan {
29
30 namespace math {
31
48 template <bool propto,
49 typename T_y, typename T_shape, typename T_scale>
50 typename return_type<T_y, T_shape, T_scale>::type
51 inv_gamma_log(const T_y& y, const T_shape& alpha, const T_scale& beta) {
52 static const char* function("stan::math::inv_gamma_log");
53 typedef typename stan::partials_return_type<T_y, T_shape, T_scale>::type
54 T_partials_return;
55
56 using stan::is_constant_struct;
57 using stan::math::check_not_nan;
58 using stan::math::check_positive_finite;
59 using boost::math::tools::promote_args;
60 using stan::math::check_consistent_sizes;
61 using stan::math::value_of;
62
63 // check if any vectors are zero length
64 if (!(stan::length(y)
65 && stan::length(alpha)
66 && stan::length(beta)))
67 return 0.0;
68
69 // set up return value accumulator
70 T_partials_return logp(0.0);
71
72 check_not_nan(function, "Random variable", y);
73 check_positive_finite(function, "Shape parameter", alpha);
74 check_positive_finite(function, "Scale parameter", beta);
75 check_consistent_sizes(function,
76 "Random variable", y,
77 "Shape parameter", alpha,
78 "Scale parameter", beta);
79
80 // check if no variables are involved and prop-to
81 if (!include_summand<propto, T_y, T_shape, T_scale>::value)
82 return 0.0;
83
84 // set up template expressions wrapping scalars into vector views
85 VectorView<const T_y> y_vec(y);
86 VectorView<const T_shape> alpha_vec(alpha);
87 VectorView<const T_scale> beta_vec(beta);
88
89 for (size_t n = 0; n < length(y); n++) {
90 const T_partials_return y_dbl = value_of(y_vec[n]);
91 if (y_dbl <= 0)
92 return LOG_ZERO;
93 }
94
95 size_t N = max_size(y, alpha, beta);
96 OperandsAndPartials<T_y, T_shape, T_scale>
97 operands_and_partials(y, alpha, beta);
98
99 using stan::math::lgamma;
100 using stan::math::digamma;
101 using std::log;
102
103 VectorBuilder<include_summand<propto, T_y, T_shape>::value,
104 T_partials_return, T_y> log_y(length(y));
105 VectorBuilder<include_summand<propto, T_y, T_scale>::value,
106 T_partials_return, T_y> inv_y(length(y));
107 for (size_t n = 0; n < length(y); n++) {
108 if (include_summand<propto, T_y, T_shape>::value)
109 if (value_of(y_vec[n]) > 0)
110 log_y[n] = log(value_of(y_vec[n]));
111 if (include_summand<propto, T_y, T_scale>::value)
112 inv_y[n] = 1.0 / value_of(y_vec[n]);
113 }
114
115 VectorBuilder<include_summand<propto, T_shape>::value,
116 T_partials_return, T_shape> lgamma_alpha(length(alpha));
117 VectorBuilder<!is_constant_struct<T_shape>::value,
118 T_partials_return, T_shape> digamma_alpha(length(alpha));
119 for (size_t n = 0; n < length(alpha); n++) {
120 if (include_summand<propto, T_shape>::value)
121 lgamma_alpha[n] = lgamma(value_of(alpha_vec[n]));
122 if (!is_constant_struct<T_shape>::value)
123 digamma_alpha[n] = digamma(value_of(alpha_vec[n]));
124 }
125
126 VectorBuilder<include_summand<propto, T_shape, T_scale>::value,
127 T_partials_return, T_scale> log_beta(length(beta));
128 if (include_summand<propto, T_shape, T_scale>::value) {
129 for (size_t n = 0; n < length(beta); n++)
130 log_beta[n] = log(value_of(beta_vec[n]));
131 }
132
133 for (size_t n = 0; n < N; n++) {
134 // pull out values of arguments
135 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
136 const T_partials_return beta_dbl = value_of(beta_vec[n]);
137
138 if (include_summand<propto, T_shape>::value)
139 logp -= lgamma_alpha[n];
140 if (include_summand<propto, T_shape, T_scale>::value)
141 logp += alpha_dbl * log_beta[n];
142 if (include_summand<propto, T_y, T_shape>::value)
143 logp -= (alpha_dbl+1.0) * log_y[n];
144 if (include_summand<propto, T_y, T_scale>::value)
145 logp -= beta_dbl * inv_y[n];
146
147 // gradients
148 if (!is_constant<typename is_vector<T_y>::type>::value)
149 operands_and_partials.d_x1[n]
150 += -(alpha_dbl+1) * inv_y[n] + beta_dbl * inv_y[n] * inv_y[n];
151 if (!is_constant<typename is_vector<T_shape>::type>::value)
152 operands_and_partials.d_x2[n]
153 += -digamma_alpha[n] + log_beta[n] - log_y[n];
154 if (!is_constant<typename is_vector<T_scale>::type>::value)
155 operands_and_partials.d_x3[n] += alpha_dbl / beta_dbl - inv_y[n];
156 }
157 return operands_and_partials.value(logp);
158 }
159
160 template <typename T_y, typename T_shape, typename T_scale>
161 inline
162 typename return_type<T_y, T_shape, T_scale>::type
163 inv_gamma_log(const T_y& y, const T_shape& alpha, const T_scale& beta) {
164 return inv_gamma_log<false>(y, alpha, beta);
165 }
166 }
167 }
168
169 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::is_constant
Metaprogramming struct to detect whether a given type is constant in the mathematical sense (not the ...
Definition: is_constant.hpp:22
lgamma.hpp
stan::math::inv_gamma_log
return_type< T_y, T_shape, T_scale >::type inv_gamma_log(const T_y &y, const T_shape &alpha, const T_scale &beta)
The log of an inverse gamma density for y with the specified shape and scale parameters.
Definition: inv_gamma_log.hpp:51
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan::is_vector::type
T type
Definition: is_vector.hpp:12
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
return_type.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
gamma_q.hpp
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
check_less_or_equal.hpp
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
grad_reg_inc_gamma.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_greater_or_equal.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.